/*
 * fd_file.c
 *
 * This file is part of the Form Designer.
 *
 * It contains the routines to save and load forms in the internal
 * format used by the form designer. This is readable ASCII.
 *
 * Written by Mark Overmars
 *
 * Version 2.1a
 * Date  Jan 24, 1992
 */

#include <stdio.h>
#include <string.h>

#include "forms.h"
#include "fd_main.h"

static void myfgets(char *line, FILE *fl)
/* My version of fgets, removing heading name:  */
{
  char tmpstr[10000];	/* Maximal label length is limited here. */
  int i = 0, j;
  int ch;
  ch = fgetc(fl);
  while ( ch != '\n' && ch != EOF) { tmpstr[i++] = ch; ch=fgetc(fl);}
  tmpstr[i] = '\0';
  i=0; while (tmpstr[i] != ':' && tmpstr[i+1] != ' ') i++;
  i += 2; j=0;
  do line[j++] = tmpstr[i++]; while (tmpstr[i-1] != '\0');
}

void save_object(FILE *fl, FL_OBJECT *obj)
/* Writes an obj description to a file. */
{
  char name[64], cbname[64], argname[64];
  get_object_name(obj,name,cbname,argname);
  fprintf(fl,"\n--------------------\n");
  fprintf(fl,"class: %d\n",obj->objclass);
  fprintf(fl,"type: %d\n",obj->type);
  fprintf(fl,"box: %f %f %f %f\n",obj->x,obj->y,obj->w,obj->h);
  fprintf(fl,"boxtype: %d\n",obj->boxtype);
  fprintf(fl,"colors: %d %d\n",obj->col1,obj->col2);
  fprintf(fl,"alignment: %d\n",obj->align);
  fprintf(fl,"style: %d\n",obj->lstyle);
  fprintf(fl,"size: %f\n",obj->lsize);
  fprintf(fl,"lcol: %d\n",obj->lcol);
  fprintf(fl,"label: %s\n",get_label(obj));
  fprintf(fl,"name: %s\n",name);
  fprintf(fl,"callback: %s\n",cbname);
  fprintf(fl,"argument: %s\n",argname);
}

static FL_OBJECT *load_object(FILE *fl)
/* Loads an object from the file and returns it. */
{
  FL_OBJECT *obj;
  int objclass,type;
  float x,y,w,h;
  int boxtype,col1,col2, lcol;
  int align,lstyle;
  float lsize;
  char label[10000];	/* Limit on label length should not be serious. */
  char name[64], cbname[64], argname[64];
  fscanf(fl,"\n--------------------\n");
  fscanf(fl,"class: %d\n",&objclass);
  fscanf(fl,"type: %d\n",&type);
  fscanf(fl,"box: %f %f %f %f\n",&x,&y,&w,&h);
  fscanf(fl,"boxtype: %d\n",&boxtype);
  fscanf(fl,"colors: %d %d\n",&col1,&col2);
  fscanf(fl,"alignment: %d\n",&align);
  fscanf(fl,"style: %d\n",&lstyle);
  fscanf(fl,"size: %f\n",&lsize);
  fscanf(fl,"lcol: %d\n",&lcol);
  myfgets(label,fl);
  myfgets(name,fl);
  myfgets(cbname,fl);
  myfgets(argname,fl);

  obj = add_an_object(objclass,type,x,y,w,h);
  if (obj == NULL)
  {
    fl_show_message("Unknown object discarded.","","");
    return NULL;
  }
  obj->col1 = col1;
  obj->col2 = col2;
  obj->boxtype = boxtype;
  obj->align = align;
  obj->lstyle = lstyle;
  obj->lsize = lsize;
  obj->lcol = lcol;
  set_label(obj,label);
  set_object_name(obj,name,cbname,argname);
  return obj;
}

void write_form(FILE *fl, FL_FORM *form, char fname[])
/* Saves a form definition to the file. */
{
  int onumb;
  FL_OBJECT *obj;
  fprintf(fl,"\n=============== FORM ===============\n");
  fprintf(fl,"Name: %s\n",fname);
  fprintf(fl,"Width: %f\n",form->w);
  fprintf(fl,"Height: %f\n",form->h);
  /* print the object number */
  onumb = 0;
  obj = form->first->next;
  while (obj != NULL) {onumb++; obj = obj->next;}
  fprintf(fl,"Number of Objects: %d\n",onumb);
  /* print the objects */
  obj = form->first->next;
  while (obj != NULL) { save_object(fl,obj); obj = obj->next; }
}

void read_form(FILE *fl, char *fname)
/* Loads a form definition from the file */
{
  float w,h;
  int onumb,i;
  fscanf(fl,"\n=============== FORM ===============\n");
  myfgets(fname,fl);
  fscanf(fl,"Width: %f\n",&w);
  fscanf(fl,"Height: %f\n",&h);
  cur_form = fl_bgn_form(FL_NO_BOX,w,h);
  fl_end_form();
  fscanf(fl,"Number of Objects: %d\n",&onumb);
  for (i=0; i<onumb;i++) load_object(fl);
}
