/*
 * fd_main.c
 * 
 * This is the main module of the Form Designer, a program that
 * helps you to design form in a fully interactive way. It
 * automatically generates the corresponding C-code.
 *
 * Written by Mark Overmars
 *
 * Version 2.1b
 * Date  Nov  4, 1992
 */

#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gl/gl.h>
#include <gl/device.h>

#include "forms.h"
#include "fd_main.h"
#include "fd_theforms.h"

long main_window = 0;	/* The main screen window */
int changed = 0;	/* Whether the thing has changed. */
char main_name[64];	/* Name of the main calling routine */

static void add_something()
/* Lets the user add some object to the form */
{
  FL_OBJECT *obj;
  float xx,yy,ww = 0.0 ,hh = 0.0 ;
  if (cur_form == NULL || cur_class == -1) return;
  winset(main_window);
  get_mouse_pos(&xx,&yy);
  scale_box(&xx,&yy,&ww,&hh);
  if (ww<2.0 || hh<2.0) return;
  obj = add_an_object(cur_class,-1,xx,yy,ww,hh);
  clear_selection();
  addto_selection(obj);
  changed = 1;
}
  
/****
  CALLBACK ROUTINES
****/

void align_cb(FL_OBJECT *obj, long arg)
/* Shows the align form. */
   {fl_show_form(alignform,FL_PLACE_POSITION,FALSE,NULL);}

void exitalign_cb(FL_OBJECT *obj, long arg)
/* Stop showing the align window */
  { fl_hide_form(alignform); }

void doalign_cb(FL_OBJECT *obj, long arg)
/* Does some alignment action */
  { align_selection( (int) arg); }
  
void snap_cb(FL_OBJECT *obj, long arg)
  { set_step_size(fl_get_counter_value(obj)); }

void func_cb(FL_OBJECT *obj, long arg)
/* The user pressed one of the function keys on the main form */
  { fl_qenter( (short) arg + F1KEY - 1, 1); }

void exit_cb(FL_OBJECT *obj, long arg)
/* Exit the program */
{
  int rep;
  if (changed)
  {
    rep = fl_show_choice("WARNING","Changes have not been saved.","",
			3,"Save","Exit","Return");
    if (rep == 1) { if (! save_forms(NULL)) return; }
    else if (rep == 3) return;
  }
  exit(0);
}

void mainname_cb(FL_OBJECT *obj, long arg)
/* Changes the main calling name. */
{
  strcpy(main_name,fl_show_input("Main creation routine name:",main_name));
  if (main_name[0] == '\0') strcpy(main_name,"create_the_forms");
  changed = 1;
}

void mergeforms_cb(FL_OBJECT *obj, long arg)
/* Merge a set of forms with the current ones. */
  { load_forms(TRUE,NULL); changed = 1; }

void saveforms_cb(FL_OBJECT *obj, long arg) 
/* Save the current set of forms. */
  { changed = ! save_forms(NULL); }

void loadforms_cb(FL_OBJECT *obj, long arg) 
/* Load a new set of forms */
{
  if (changed)
  {
    if (fl_show_question("Forms have been changed!","Should I save them?",""))
      { if (!save_forms(NULL)) return; }
  }
  load_forms(FALSE,NULL);
  changed = 0;
}

static FL_FORM *thetestform;	/* the test form */

void test_cb(FL_OBJECT *obj, long arg)
/* Start testing the current form. */
{
  if (cur_form == NULL) return;
  fl_deactivate_form(controlform);
  thetestform = cur_form;
  cur_form = NULL;
  redraw_the_form(1);
  thetestform->doublebuf = fl_doublebuf;
  fl_clear_browser(testbrowser);
  fl_show_form(thetestform,FL_PLACE_CENTER,FALSE,NULL);
  fl_show_form(testform,FL_PLACE_POSITION,FALSE,NULL);
}

void stoptest_cb(FL_OBJECT *obj, long arg)
/* Ends a testing session */
{
  if (thetestform == NULL) return;
  fl_hide_form(testform);
  fl_hide_form(thetestform);
  thetestform->doublebuf = FALSE;
  cur_form = thetestform;
  thetestform = NULL;
  redraw_the_form(0);
  fl_activate_form(controlform);
}

void main_loop()
/* The main event handling loop. */
{
  int s;
  long dev;
  short val;
  FL_OBJECT *obj;
  char str[128], name[128], cbname[128], argname[128];
  while (1)
  {
    obj = fl_do_forms();
    s = (getbutton(LEFTSHIFTKEY) || getbutton(RIGHTSHIFTKEY));

    /* Check whether testing a form */
    if (thetestform != NULL && obj != FL_EVENT)
    {
      get_object_name(obj,name,cbname,argname);
      if (name[0] == '\0') strcpy(name,"<noname>");
      if (cbname[0] != '\0')
        sprintf(str,"Called: %s(%s,%s);",cbname,name,argname);
      else
        sprintf(str,"Returned: %s",name);
      fl_addto_browser(testbrowser,str);
    }

    /* Check for events to treat */
    if (fl_qtest())
    {
      fl_deactivate_form(controlform);
      dev = fl_qread(&val);
      winset(main_window);
      if (val)
        switch (dev)
        {
          case F1KEY:  change_selection(); redraw_the_form(0);break;
          case F2KEY:  lower_selection(); redraw_the_form(0);break;
          case F3KEY:  raise_selection(); redraw_the_form(0);break;
          case F4KEY:  select_all(); redraw_the_form(0);break;

	  case F5KEY:  show_selection(); redraw_the_form(0);break;
	  case F6KEY:  hide_selection(); redraw_the_form(0);break;
          case F7KEY:  group_selection(); break;
          case F8KEY:  flatten_selection(); break;

          case F9KEY:  copy_selection(); break;
          case F10KEY: paste_selection(); redraw_the_form(0);break;
	  case F11KEY: break;
          case F12KEY: cut_selection(); redraw_the_form(0);break;

          case MOUSE3: if (s) {add_something(); redraw_the_form(0);} break;
          case MOUSE2: handle_move(); redraw_the_form(0); break;
          case MOUSE1: handle_select(); redraw_the_form(0); break;

          case REDRAW: redraw_the_form(1); break;
        }
      fl_activate_form(controlform);
    }
  }
}

main(int ac, char *av[])
{
  long w = -1 ,h = -1;
  int c;
  int i;
  char *options = "w:h:c:";

  /* Init forms */
  fl_init();

  /* Create the forms */
  create_the_forms();
  fl_set_form_position(testform,-1,-1);
  fl_set_form_position(alignform,-1,-1);
  fl_set_form_position(helpform,-1,421);
  fl_set_form_position(controlform,-1,1);

  /* Initialize stuff */
  init_classes();
  set_step_size(10.0);
  fl_set_counter_bounds(snapobj,0.0,1000.0);
  fl_set_counter_step(snapobj,1.0,5.0);
  fl_set_counter_value(snapobj,10.0);
  fl_set_counter_precision(snapobj,0);
  showhelp_cb(NULL,1);
  thetestform = NULL;
  strcpy(main_name,"create_the_forms");

  /* get all the option flags from the command line */
  opterr = 0;
  while ((c=getopt(ac,av,options)) != EOF)
    switch (c)
  {
    case 'h'  : if ((c=atoi(optarg)) != 0) h = c; break;
    case 'w'  : if ((c=atoi(optarg)) != 0) w = c; break;
    case 'c'  :
    	load_forms(FALSE, optarg);
	save_forms(optarg);
	exit(0);
	break;
    case '?'  : 
    default   :
	fprintf(stderr,
             "Usage: %s [-w max_form_width -h max_form_height] [<fname>]\n\
          or\n       %s -c <fname>\n", *av, *av);
	exit(0);
        break;
  }

  /* Create main window */
  if (w > 0 && h > 0) prefsize(w, h);
  main_window = winopen("Form Design");
  if (fl_doublebuf) doublebuffer();
  if (fl_rgbmode) RGBmode();
  gconfig();
  fl_qdevice(MOUSE1);
  fl_qdevice(MOUSE2);
  fl_qdevice(MOUSE3);
  for (i=F1KEY; i<=F12KEY; i++) fl_qdevice(i);

  /* Show forms window */
  redraw_the_form(1);
  fl_show_form(controlform,FL_PLACE_POSITION,FALSE,NULL);

  /* Load a file */
  if (optind < ac)
  {
    load_forms(FALSE, av[optind++]);
    while (optind < ac) load_forms(TRUE, av[optind++]);
  }

  /* Main loop */
  main_loop();
}
