/*
 * fd_print.c
 *
 * This file is part of the Form Designer.
 *
 * It contains the routines to load and save form definitions,
 * C-format. (It is quite messy!)
 *
 * Written by Mark Overmars
 *
 * Version 2.0a
 * Date  Nov  2, 1991
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "forms.h"
#include "fd_main.h"

/*------------- Keeping track of array names. -------------------*/

#define MAXARNAME	100

char *arnames[MAXARNAME];
int arsizes[MAXARNAME];
int  anumb = 0;

void init_array_names()
/* Initializes the aray names. */
  { anumb = 0; }

int check_array_name(char aname[])
/* Checks whether this name is an array name and remembers it */
{
  char tmpstr[64];
  int i,j,ind;
  strcpy(tmpstr,aname);
  for (i=0; tmpstr[i] != '[' && tmpstr[i] != NULL; i++) ;
  if (tmpstr[i] == NULL) return FALSE;
  ind = 0;
  for (j= i+1; tmpstr[j] != ']' && tmpstr[j] != NULL; j++)
    if (tmpstr[j] >= '0' && tmpstr[j] <= '9')
      ind = 10*ind + tmpstr[j] - '0';
  tmpstr[i] = NULL;
  for (i=0; i<anumb; i++)
    if (strcmp(arnames[i],tmpstr) == 0)
      {if (ind+1 > arsizes[i]) arsizes[i] = ind + 1; return TRUE;}
  if (anumb == MAXARNAME) return FALSE;
  arnames[anumb] = (char *) malloc(64);
  strcpy(arnames[anumb],tmpstr);
  arsizes[anumb] = ind +1;
  anumb++;
  return TRUE;
}

int arethere_array_names()
  { return (anumb > 0); }

void print_array_names(FILE *fn)
/* Prints the array names of the file. */
{
  int i;
  for (i=0; i<anumb; i++)
  {
    fprintf(fn,"        *%s[%i]",arnames[i],arsizes[i]);
    if (i < anumb-1) fprintf(fn,",\n");
  }
}

/*----------------------- Printing the C-code --------------------*/

void print_form(FILE *fn, FL_FORM *form, char fname[])
/* Prints the form description in C-code */
{
  FL_OBJECT *obj, *defobj;
  char name[40], cbname[40], argname[40];
  int first = 1;
  /* print the form declaration */
  fprintf(fn,"FL_FORM *%s;\n\n",fname);
  /* print the object declarations */
  init_array_names();
  obj = form->first;
  while ( (obj = obj->next) != NULL)
  {
    get_object_name(obj,name,cbname,argname);
    if (name[0] != 0) 
    {
      if (!check_array_name(name))
      {
	if (first) fprintf(fn,"FL_OBJECT\n"); else fprintf(fn,",\n");
	first = 0;
	fprintf(fn,"        *%s",name);
      }
    }
  }
  if (arethere_array_names())
  {
    if (first) fprintf(fn,"FL_OBJECT\n"); else fprintf(fn,",\n");
    first = 0;
    print_array_names(fn);
  }
  if (!first) fprintf(fn,";\n\n"); else fprintf(fn,"\n");
  /* Print the defining procedure */
  fprintf(fn,"void create_form_%s(void)\n{\n  FL_OBJECT *obj;\n",fname);
  fprintf(fn,"  %s = fl_bgn_form(FL_NO_BOX,%.1f,%.1f);\n",fname,form->w,form->h);
  obj = form->first;
  while ( (obj = obj->next) != NULL)
  {
    get_object_name(obj,name,cbname,argname);
    if (obj->objclass == FL_BEGIN_GROUP)
    {
      if (name[0] != '\0') fprintf(fn,"  %s = ",name); else fprintf(fn,"  ");
      fprintf(fn,"fl_bgn_group();\n");
    }
    else if (obj->objclass == FL_END_GROUP)
    {
      fprintf(fn,"  fl_end_group();\n");
    }
    else
    {
      defobj = find_class_default(obj->objclass);
      fprintf(fn,"  ");
      if (name[0] != '\0') fprintf(fn,"%s = ",name);
      fprintf(fn,"obj = ");
      fprintf(fn,"fl_add_%s(",find_class_name(obj->objclass));
      fprintf(fn,"FL_%s,",find_type_name(obj->objclass,obj->type));
      fprintf(fn,"%.1f,%.1f,%.1f,%.1f,\"%s\");\n",
                obj->x, obj->y, obj->w, obj->h, get_label(obj));
      if (obj->boxtype != defobj->boxtype && obj->objclass != FL_BOX)
      {
        if (obj->boxtype == FL_NO_BOX)
          fprintf(fn,"    fl_set_object_boxtype(obj,FL_NO_BOX);\n");
        else if (obj->boxtype == FL_UP_BOX)
          fprintf(fn,"    fl_set_object_boxtype(obj,FL_UP_BOX);\n");
        else if (obj->boxtype == FL_DOWN_BOX)
          fprintf(fn,"    fl_set_object_boxtype(obj,FL_DOWN_BOX);\n");
        else if (obj->boxtype == FL_FLAT_BOX)
          fprintf(fn,"    fl_set_object_boxtype(obj,FL_FLAT_BOX);\n");
        else if (obj->boxtype == FL_BORDER_BOX)
          fprintf(fn,"    fl_set_object_boxtype(obj,FL_BORDER_BOX);\n");
        else if (obj->boxtype == FL_SHADOW_BOX)
          fprintf(fn,"    fl_set_object_boxtype(obj,FL_SHADOW_BOX);\n");
        else if (obj->boxtype == FL_FRAME_BOX)
          fprintf(fn,"    fl_set_object_boxtype(obj,FL_FRAME_BOX);\n");
        else if (obj->boxtype == FL_ROUNDED_BOX)
          fprintf(fn,"    fl_set_object_boxtype(obj,FL_ROUNDED_BOX);\n");
        else if (obj->boxtype == FL_RFLAT_BOX)
          fprintf(fn,"    fl_set_object_boxtype(obj,FL_RFLAT_BOX);\n");
        else if (obj->boxtype == FL_RSHADOW_BOX)
          fprintf(fn,"    fl_set_object_boxtype(obj,FL_RSHADOW_BOX);\n");
      }
      if (obj->col1 != defobj->col1 || obj->col2 != defobj->col2)
        fprintf(fn,"    fl_set_object_color(obj,%d,%d);\n",obj->col1,obj->col2);
      if (obj->lcol != defobj->lcol)
        fprintf(fn,"    fl_set_object_lcol(obj,%d);\n", obj->lcol);
      if (obj->lsize != defobj->lsize)
      {
        if (obj->lsize == FL_SMALL_FONT)
          fprintf(fn,"    fl_set_object_lsize(obj,FL_SMALL_FONT);\n");
        else if (obj->lsize == FL_NORMAL_FONT)
          fprintf(fn,"    fl_set_object_lsize(obj,FL_NORMAL_FONT);\n");
        else if (obj->lsize == FL_LARGE_FONT)
          fprintf(fn,"    fl_set_object_lsize(obj,FL_LARGE_FONT);\n");
        else
          fprintf(fn,"    fl_set_object_lsize(obj,%f);\n",obj->lsize);
      }
      if (obj->align != defobj->align)
      {
        if (obj->align == FL_ALIGN_LEFT)
          fprintf(fn,"    fl_set_object_align(obj,FL_ALIGN_LEFT);\n");
        else if (obj->align == FL_ALIGN_RIGHT)
          fprintf(fn,"    fl_set_object_align(obj,FL_ALIGN_RIGHT);\n");
        else if (obj->align == FL_ALIGN_TOP)
          fprintf(fn,"    fl_set_object_align(obj,FL_ALIGN_TOP);\n");
        else if (obj->align == FL_ALIGN_BOTTOM)
          fprintf(fn,"    fl_set_object_align(obj,FL_ALIGN_BOTTOM);\n");
        else if (obj->align == FL_ALIGN_CENTER)
          fprintf(fn,"    fl_set_object_align(obj,FL_ALIGN_CENTER);\n");
      }
      if (obj->lstyle != defobj->lstyle)
      {
        if (obj->lstyle == FL_NORMAL_STYLE)
          fprintf(fn,"    fl_set_object_lstyle(obj,FL_NORMAL_STYLE);\n");
        else if (obj->lstyle == FL_BOLD_STYLE)
          fprintf(fn,"    fl_set_object_lstyle(obj,FL_BOLD_STYLE);\n");
        else if (obj->lstyle == FL_ITALIC_STYLE)
          fprintf(fn,"    fl_set_object_lstyle(obj,FL_ITALIC_STYLE);\n");
        else if (obj->lstyle == FL_FIXED_STYLE)
          fprintf(fn,"    fl_set_object_lstyle(obj,FL_FIXED_STYLE);\n");
        else
          fprintf(fn,"    fl_set_object_lstyle(obj,%d);\n",obj->lstyle);
      }
      if (cbname[0] != 0)
        fprintf(fn,"    fl_set_call_back(obj,%s,%s);\n",cbname,argname);
    }
  }
  fprintf(fn,"  fl_end_form();\n}\n\n");
  fprintf(fn,"/*---------------------------------------*/\n\n");
}

void print_callbacks(FILE *fn, FL_FORM *form)
/* Prints the callback routines used in form */
{
  FL_OBJECT *obj;
  char name[40], cbname[40], argname[40];
  obj = form->first;
  while ( (obj = obj->next) != NULL)
  {
    get_object_name(obj,name,cbname,argname);
    if (cbname[0] != 0) fprintf(fn,"extern void %s(FL_OBJECT *, long);\n",cbname);
  }
  fprintf(fn,"\n");
}

void print_objects(FILE *fn, FL_FORM *form, char fname[])
/* Prints the objects declarations used in form */
{
  FL_OBJECT *obj;
  char name[40], cbname[40], argname[40];
  int first = 1;
  fprintf(fn,"extern FL_FORM *%s;\n\n",fname);
  init_array_names();
  obj = form->first;
  while ( (obj = obj->next) != NULL)
  {
    get_object_name(obj,name,cbname,argname);
    if (name[0] != 0) 
    {
      if (!check_array_name(name))
      {
	if (first) fprintf(fn,"extern FL_OBJECT\n"); else fprintf(fn,",\n");
	first = 0;
	fprintf(fn,"        *%s",name);
      }
    }
  }
  if (arethere_array_names())
  {
    if (first) fprintf(fn,"extern FL_OBJECT\n"); else fprintf(fn,",\n");
    first = 0;
    print_array_names(fn);
  }
  if (!first) fprintf(fn,";\n\n");
}
